--world.lua
-- Holds all game logic and methods related to the world

world = {}


world_meta = {__index = world}

local color = {
		cell = {
				alive = {255, 255, 255, 255},
				dead = {30, 30, 30, 255}
			}
	}

function world.new(x, y, w, h, cellSize)
	local world = {
			x = x,
			y = y,
			w = w,
			h = h,
			cellSize = cellSize,
			n = 0,
			cell = {}
		}
	
	for y=0, h-1 do
		world.cell[y] = {}
		for x=0, w-1 do
			world.cell[y][x] = {state = {c = false, n = false}, n = 0}
		end
	end
	
	return setmetatable(world, world_meta)
end


function world:draw()
	for y=0, self.h-1 do
		for x=0, self.w-1 do
			if self.cell[y][x].state.c == true then
				love.graphics.setColor(color.cell.alive)
			else	
				love.graphics.setColor(color.cell.dead)
			end
			
			love.graphics.rectangle("fill", self.x + x * self.cellSize, self.y + y * self.cellSize, self.cellSize-1, self.cellSize-1)
		end
	end
end


--CELL METHODS
function world:setCellState(x, y, state)
	self.cell[y][x].state.c = state
end

function world:getWorldMouse()
	local x = math.floor((love.mouse.getX() - self.x) / self.cellSize)
	local y = math.floor((love.mouse.getY() - self.y) / self.cellSize)
	
	if x > self.w then
		x = self.w
	elseif x < 0 then
		x = 0
	end
	
	if y > self.h then
		y = self.h
	elseif y < 0 then
		y = 0
	end
	
	return x, y
end


function world:update()
	--Individual cell update
	for y=0, self.h-1 do
		for x=0, self.w-1 do	
			local check = {
					{x = x + 1, y = y},
					{x = x - 1, y = y},
					{x = x, y = y + 1},
					{x = x, y = y - 1},
					{x = x - 1, y = y + 1},
					{x = x + 1, y = y - 1},
					{x = x + 1, y = y + 1},
					{x = x - 1, y = y - 1}
				}
			local n = 0 -- Cell Neighbours
			for i,v in ipairs(check) do
				if v.x >= 0 and v.x <= self.w-1 and v.y >= 0 and v.y <= self.h-1 then
					if self.cell[v.y][v.x].state.c == true then
						n = n + 1
					end
				end
			end			
		
			local state = false
			if self.cell[y][x].state.c == true then -- Live cell						
				if n == 2 or n == 3 then
					state = true
				else
					state = false
				end
			else -- Dead cell
				if n == 3 then
					state = true
				else
					state = false
				end
			end
			self.cell[y][x].state.n = state
		end
	end
	
	--Entire Map Update
	for y=0, self.h-1 do
		for x=0, self.w-1 do	
			self:setCellState(x, y, self.cell[y][x].state.n)
		end
	end
end


















